<?php

// Copyright Nikki Moreaux 2014-2022 https://nmoreaux.com/projects/img/
// Corrigé par Romain Giovanetti https://giovanetti.fr/

ini_set('memory_limit', '-1');
$debug = false;
$cache_folder = '_img_cache';


if($debug){
	print('<pre>');
}

function error($error){
	global $debug,$settings;

	if($debug){
		print($error);
		exit();
	}

	$width = max($settings['width'], 600);
	$height = max($settings['height'], 20);
	$image = imagecreatetruecolor($width, $height);
	$white = imagecolorallocate($image, 255, 255, 255);
	imagefill($image, 0, 0, $white);
	$red = imagecolorallocate($image, 255, 20, 20);
	imagestring($image, 5, 5, 5, $error, $red);


	header('Content-type: image/png');
	imagepng($image);
	imagedestroy($image);
	exit();
}
function error_handler($errno, $errstr, $errfile, $errline, $errcontext){
	error($errstr);
}
set_error_handler('error_handler');



// Set a PNGQuant instance for faster loadings; buy yours from Nikki Moreaux via contact@nmoreaux.com
$pngquant_url = false;


$settings = array(
	'file' => 			false,
	'width' => 			false,
	'height' => 		false,
	'pixel_ratio' => 	1,
	'format' => 		false,
	'quality' => 		70,
	'brightness' => 	0,
	'grayscale' =>		0,
	'fill' =>			0,
	'fill_transparent' => false,
	'fill_color' =>		'ffffff',
	'convolution' => 	0,
	'even_rounding' =>	false,
	'pngquant' => 		true
);


// Extract settings from query string
$settings_doublet = explode(',', str_replace('&', ',', $_SERVER['QUERY_STRING']));

foreach ($settings_doublet as $doublet) {
	$kv = explode('=', $doublet);

	if($kv[0] == 'w')
		$kv[0] = 'width';
	elseif($kv[0] == 'h')
		$kv[0] = 'height';
	elseif($kv[0] == 'pr')
		$kv[0] = 'pixel_ratio';
	elseif($kv[0] == 'format' && $kv[1] == 'jpeg')
		$kv[1] = 'jpg';

	if(count($kv) == 1){
		if($kv[0] == 'grayscale'){
			array_push($kv, 1);
		}if($kv[0] == 'fill_transparent'){
			array_push($kv, 1);
		}else{
			array_unshift($kv, 'file');
		}
	}

	if($kv[0] == 'width' || $kv[0] == 'height'){
		if(is_string($kv[1])){
			$kv[1] = preg_replace("/[^0-9]/","",$kv[1]);
		}
	}


	if(array_key_exists($kv[0], $settings)){
		$settings[$kv[0]] = $kv[1];
	}
}


// Cache / to check if query already requested
$cache_path = md5($settings['file']).'_'.md5($_SERVER['QUERY_STRING']).'_'.md5($_SERVER['QUERY_STRING'].'salt');
$cache_format = false;
$cache_file = false;
$cache_time = 0;
$last_modification_time = 1;
clearstatcache();

if(is_readable($cache_folder.'/'.$cache_path.'.png')){
	$cache_format = 'png';
}
if(is_readable($cache_folder.'/'.$cache_path.'.jpg')){
	$cache_format = 'jpg';
}
if(is_readable($cache_folder.'/'.$cache_path.'.svg')){
	$cache_format = 'svg';
}

if($cache_format){
	$cache_file = $cache_folder.'/'.$cache_path.'.'.$cache_format;
	$cache_time = filemtime($cache_file);
}

// Check if file exists
if(!file_exists($settings['file'])){
	error('file '.$settings['file'].' do not exists');
}else{
	 $last_modification_time = max($last_modification_time,filemtime($settings['file']));
}

// If the cache file is too old or don't exists, build it
if($cache_time < $last_modification_time){

	// Special catch for SVG files
	if(substr($settings['file'],-4,4) === '.svg'){

		$svg_contents = file_get_contents($settings['file']);
		if($settings['fill_transparent'] == 1 && $settings['fill_color']){
			$svg_contents = preg_replace('/\#.+?\"/','#'.$settings['fill_color'].'"',$svg_contents);
		}

		$cache_format = 'svg';
		$cache_file = $cache_folder.'/'.$cache_path.'.'.$cache_format;
		file_put_contents($cache_file,$svg_contents);
		$cache_time = filemtime($cache_file);

	}else{

		// Check if file is an image
		$im_size = getimagesize($settings['file']);
		if($im_size === false || substr($im_size['mime'], 0, 5) !== 'image'){
			error('file '.$settings['file'].' is not an image');
		}


		// Set output format if not already set
		if($settings['format'] === false){
			$settings['format'] = substr($im_size['mime'], 6);
			if($settings['format'] != 'jpg' && $settings['format'] != 'png'){
				$settings['format'] = 'jpg';
			}
		}


		// Set W&&H format if not already set, set W||H if the other is set
		$source_image_left_crop = 0;
		$source_image_top_crop = 0;
		if($settings['width'] === false && $settings['height'] === false){
			$settings['width'] = $im_size[0];
			$settings['height'] = $im_size[1];
		}else{
			if($settings['width'] === false){
				$settings['width'] = $im_size[0]/($im_size[1]/$settings['height']);
				if($settings['even_rounding'])
					$settings['width'] = round($settings['width']/2)*2;
			}
			if($settings['height'] === false){
				$settings['height'] = $im_size[1]/($im_size[0]/$settings['width']);
				if($settings['even_rounding'])
					$settings['height'] = round($settings['height']/2)*2;
			}

			$source_ratio = $im_size[0]/$im_size[1];
			$output_ratio = $settings['width']/$settings['height'];

			if($source_ratio > $output_ratio){
				$source_image_left_crop = (($source_ratio-$output_ratio)/$source_ratio)*$im_size[0]/2;
			}
			if($source_ratio < $output_ratio){
				$source_image_top_crop = (($output_ratio-$source_ratio)/$output_ratio)*$im_size[1]/2;
			}
		}


		// Create image and copy the cropped source
		$source_image = imagecreatefromstring(file_get_contents($settings['file']));


		$output_width = (int)($settings['width']*$settings['pixel_ratio']);
		$output_height = (int)($settings['height']*$settings['pixel_ratio']);

		$image = imagecreatetruecolor(
			$output_width, $output_height
		);

		if($settings['format'] == 'png'){
			imagealphablending($image, false);
			imagesavealpha($image, true);
			$transp = imagecolorallocatealpha($image, 255, 255, 255, 127);
			imagefill($image, 0, 0, $transp);
		}else{
			$white = imagecolorallocate($image, 255, 255, 255);
			imagefill($image, 0, 0, $white);
		}

		imagecopyresampled($image, $source_image, 0, 0, $source_image_left_crop, $source_image_top_crop,
		$output_width, $output_height,
		(int)($im_size[0]-($source_image_left_crop*2)), (int)($im_size[1]-($source_image_top_crop*2)));
		imagedestroy($source_image);


		// Filters !
		if($settings['grayscale'] > 0){
			if($settings['grayscale'] == 1){
				imagefilter($image, IMG_FILTER_GRAYSCALE);
			}else{
				$comp = imagecreatetruecolor(
					$output_width, $output_height
				);

				imagecopy($comp, $image, 0, 0, 0, 0, $output_width, $output_height);

				imagefilter($comp, IMG_FILTER_GRAYSCALE);

				imagecopymerge($image, $comp, 0, 0, 0, 0, $output_width, $output_height, $settings['grayscale']*100);

				imagedestroy($comp);
			}
		}


		if($settings['convolution']){
			// Sharpen
			if($settings['convolution']<0){
				for ($i=0; $i > floor($settings['convolution']); $i--) {
					$matrix = array
					(
						array(-1.2, -1, -1.2),
						array(-1, 20, -1),
						array(-1.2, -1, -1.2)
					);
					$divisor = array_sum(array_map('array_sum', $matrix));
					imageconvolution($image, $matrix, $divisor, 0);
				}
			}
			// Blur
			if($settings['convolution']>0){
				if($settings['convolution']>1){
					$settings['convolution'] = min($settings['convolution'],20);
					// Ugly method... extension_loaded('imagick')
					if(false) {

						ob_start();

						imagepng($image);
						imagedestroy($image);

						$imagick = new Imagick();
						$imagick->$imagick(ob_get_contents());
						ob_end_clean();

						$imagick->gaussianBlurImage($settings['convolution'],$settings['convolution']/2);

						$image = imagecreatefromstring($imagick);

						$imagick->destroy();
					}else{
						for ($i=0; $i < ceil($settings['convolution']); $i++) {
							imagefilter($image, IMG_FILTER_GAUSSIAN_BLUR);
						}
					}
				}
				imagefilter($image, IMG_FILTER_GAUSSIAN_BLUR);
			}
		}


		if($settings['brightness'] !== 0){
			imagefilter($image, IMG_FILTER_BRIGHTNESS, 255*$settings['brightness']);
		}

		if($settings['fill'] !== 0){
			$fill_image = imagecreatetruecolor($output_width, $output_height);

			if($settings['fill'] > 0){

				if(strlen($settings['fill_color']) == 7){
					list($r, $g, $b) = sscanf($settings['fill_color'], '#%02x%02x%02x');
				}else{
					list($r, $g, $b) = sscanf($settings['fill_color'], '%02x%02x%02x');
				}

				$color = imagecolorallocate($fill_image, $r, $g, $b);
			}else{
				$color = imagecolorallocate($fill_image, 0, 0, 0);
			}
			imagefill($fill_image, 0, 0, $color);

			imagecopymerge($image, $fill_image, 0, 0, 0, 0, $output_width, $output_height, abs($settings['fill'])*100);

			imagedestroy($fill_image);
		}

		if($settings['fill_transparent'] == 1){

			if(strlen($settings['fill_color']) == 7){
				$new_rgb = sscanf($settings['fill_color'], '#%02x%02x%02x');
			}else{
				$new_rgb = sscanf($settings['fill_color'], '%02x%02x%02x');
			}

			$w = imagesx($image);
			$h = imagesy($image);

			// Work through pixels
			for($y=0;$y<$h;$y++) {
				for($x=0;$x<$w;$x++) {
					// Apply new color + Alpha
					$rgb = imagecolorsforindex($image, imagecolorat($image, $x, $y));

					$pixelColor = imagecolorallocatealpha($image, $new_rgb[0], $new_rgb[1], $new_rgb[2], $rgb['alpha']);
					imagesetpixel($image, $x, $y, $pixelColor);
				}
			}

			// Restore Alpha
			imageAlphaBlending($image, true);
			imageSaveAlpha($image, true);
		}


		// Output
		if($debug){
			print_r($settings);
		}else{

			if (!file_exists($cache_folder)) {
				@mkdir($cache_folder);
			}

			if($settings['format'] == 'png'){
				$cache_format = 'png';
				$cache_file = $cache_folder.'/'.$cache_path.'.'.$cache_format;
				imagepng($image,$cache_file,9,PNG_ALL_FILTERS);


				if($pngquant_url && $settings['pngquant'] && $settings['pngquant'] !== 'false'){

					// Try to pngquant; not looped since we already wrote the png
					$query = http_build_query(array(
						'url' => 'http' . (isset($_SERVER['HTTPS']) ? 's' : '') . '://' . "{$_SERVER['HTTP_HOST']}/{$_SERVER['REQUEST_URI']}",
						'quality' => $settings['quality']
					));
					$server_output = file_get_contents($pngquant_url.'?'.$query);

					//file_put_contents('logs.txt', $_SERVER['REQUEST_URI'].PHP_EOL , FILE_APPEND | LOCK_EX);

					$pngquant_image = @imagecreatefromstring($server_output);
					if($pngquant_image !== false){
						file_put_contents($cache_file,$server_output);
					}

				}

			}else{
				$cache_format = 'jpg';
				$cache_file = $cache_folder.'/'.$cache_path.'.'.$cache_format;
				imagejpeg($image,$cache_file,$settings['quality']);
			}

			$cache_time = filemtime($cache_file);

		}
		imagedestroy($image);
	}
}



// Serve the cached file

$gmt_mtime = gmdate('r',$cache_time);
$etag = md5($cache_time.$cache_file.filesize($cache_file));
header('ETag: "'.$etag.'"');
header('Last-Modified: '.$gmt_mtime);
header('Cache-Control: no-cache');

$is_in_client_cache = false;

if(isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) && $_SERVER['HTTP_IF_MODIFIED_SINCE'] == $gmt_mtime) {
	$is_in_client_cache = true;
}
if(isset($_SERVER['HTTP_IF_NONE_MATCH']) && str_replace('"', '', stripslashes($_SERVER['HTTP_IF_NONE_MATCH'])) == $etag){
	$is_in_client_cache = true;
}

if($is_in_client_cache){
    header('HTTP/1.1 304 Not Modified');
    exit();
}

if($cache_format == 'png'){
	header('Content-Type: image/png');
	readfile($cache_file);
}else if($cache_format == 'svg'){
	header('Content-Type: image/svg+xml');
	readfile($cache_file);
}else{
	header('Content-Type: image/jpeg');
	readfile($cache_file);
}


?>
